package dolmen_type

  1. Overview
  2. Docs

Module Real.TffSource

Parameters

module Type : Tff_intf.S

Signature

Sourcetype Type.warn +=
  1. | Restriction : string -> Dolmen.Std.Term.t Type.warn
    (*

    Warning for expressions which tecnically do not respect the strict spec but respect the large spec.

    *)

Arithmetic type-checking warnings

Sourcetype Type.err +=
  1. | Forbidden : string -> Dolmen.Std.Term.t Type.err
    (*

    Error for expressions which do not respect the spec.

    *)

Arithmetic type-checking errors

Parsing function for type-checking