package dolmen_type

  1. Overview
  2. Docs

Module Dolmen_type.IntfSource

External Typechecker interface

This module defines the external typechcker interface, that is, the interface of an instantiated typechecker.

Typechecker interface

Sourcetype symbol =
  1. | Id of Dolmen.Std.Id.t
  2. | Builtin of Dolmen.Std.Term.builtin

Wrapper around potential function symbols from the Dolmen AST.

Sourcemodule type Formulas = sig ... end

Typechecker interface