package dolmen_type

  1. Overview
  2. Docs

Module Dolmen_type.MiscSource

Misc

Collection of various helper functions, some rather specific to dolmen (such as the bitv manipulation module), and others more generic (e.g. the

ist

and Option modules. For these generic functions, they are here either because they do not exist in the stdlib and they did not (yet) warrant depending on an external alternative stdlib, or because they were added in recent versions of ocaml and thus would unnecessarily restrict the availability of dolmen on older versions of ocaml.

Sourcemodule Options : sig ... end

Option helpers

Sourcemodule Lists : sig ... end

List helpers

Sourcemodule Strings : sig ... end

String helper

Sourcemodule Bitv : sig ... end

Bitvector helpers